class TestPopulate:

    def test_populate_about(self, db_session, root):
        assert 'about' in root

    def test_populate_acl(self, db_session, root):
        allow_everyone_view = [item for item in root.__acl__
                               if item[0] == 'Allow' and
                               item[1] == 'System.everyone' and
                               item[2] == ['view']]
        assert allow_everyone_view == []



from pyramid.i18n import LocalizerRequestMixin
from pyramid.threadlocal import get_current_registry
from kotti.populate import (
    populate_users,
    _ROOT_ATTRS,
    _ABOUT_ATTRS,
    )
from kotti import (
    DBSession,
    get_settings,
    )
from kotti.resources import (
    Document,
    Node,
    )
from kotti_backend.security import SITE_ACL


def populate():
    """
    Create the root node (:class:`~kotti.resources.Document`) and the 'about'
    subnode in the nodes tree if there are no nodes yet.
    """
    lrm = LocalizerRequestMixin()
    lrm.registry = get_current_registry()
    lrm.locale_name = get_settings()['pyramid.default_locale_name']
    localizer = lrm.localizer

    if DBSession.query(Node.id).count() == 0:
        localized_root_attrs = dict(
            [(k, localizer.translate(v)) for k, v in _ROOT_ATTRS.iteritems()])
        root = Document(**localized_root_attrs)
        root.__acl__ = SITE_ACL
        DBSession.add(root)
        localized_about_attrs = dict(
            [(k, localizer.translate(v)) for k, v in _ABOUT_ATTRS.iteritems()])
        root['about'] = Document(**localized_about_attrs)
        DBSession.flush()

    populate_users()



from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound

from kotti.interfaces import IContent
from kotti.resources import get_root


@view_config(
    name='goto_frontend',
    context=IContent,
    if_setting_has_value=('kotti_backend.goto_frontend', True),
    permission='view',
    )
def goto_frontend_view(request):
    settings = request.registry.settings
    not_publishable_types = settings.get('kotti_backend.not_publishable_types',
                                         ()
                                        )
    context = request.context
    parent = context.parent
    while context.type_info.name in not_publishable_types and parent is not None:
        context = context.parent
    script_name = request.environ.get('SCRIPT_NAME')
    frontend_url = settings.get('kotti_backend.frontend_url', None)
    app_url = None
    if frontend_url is not None:
        # use a custom frontend url
        app_url = frontend_url
    elif script_name is not None:
        # default setup with script name (default: /cms)
        root = get_root()
        app_url = ''.join(request.resource_url(root).rsplit(script_name)).rstrip('/')
    return HTTPFound(
        request.resource_url(
            context,
            app_url=app_url,
            )
        )


def includeme(config):
    config.scan(__name__)



from kotti.testing import user


class TestGotoFrontend:

    def dummy_request(self, context, settings, environ):
        from kotti.testing import DummyRequest
        dummy_request = DummyRequest()
        dummy_request.registry.settings = settings
        dummy_request.environ = environ
        dummy_request.context = context

        _resource_url = dummy_request.resource_url
        dummy_request._resource_url = _resource_url

        def resource_url(context, *args, **kwargs):
            app_url = kwargs.pop('app_url', None)
            if app_url is not None:
                return dummy_request._resource_url(context, app_url=app_url)
            else:
                return dummy_request._resource_url(
                    context,
                    app_url='http://example.com{0}'.format(environ['SCRIPT_NAME']),
                    *args,
                    **kwargs
                    )
        dummy_request.resource_url = resource_url
        return dummy_request

    def _create_file(self, config):
        from kotti.resources import File
        return File("file contents", u"myf\xfcle.png", u"image/png")

    def test_goto_frontend_default(self, root, content):
        from kotti_backend.views.goto_frontend import goto_frontend_view
        from pyramid.httpexceptions import HTTPFound
        settings = {}
        environ = {'SCRIPT_NAME': '/cms'}
        dummy_request = self.dummy_request(root['about'], settings, environ)

        assert dummy_request.resource_url(root['about']) == 'http://example.com/cms/about/'

        import mock
        with mock.patch('kotti_backend.views.goto_frontend.get_root') as get_root:
            get_root.return_value = root
            resp = goto_frontend_view(dummy_request)

        assert resp.location == 'http://example.com/about/'
        assert isinstance(resp, HTTPFound)

    def test_goto_frontend_custom_url(self, root, content):
        from kotti_backend.views.goto_frontend import goto_frontend_view
        from pyramid.httpexceptions import HTTPFound
        settings = {'kotti_backend.frontend_url': 'http://google.com'}
        environ = {'SCRIPT_NAME': '/cms'}
        dummy_request = self.dummy_request(root['about'], settings, environ)

        assert dummy_request.resource_url(root['about']) == 'http://example.com/cms/about/'

        import mock
        with mock.patch('kotti_backend.views.goto_frontend.get_root') as get_root:
            get_root.return_value = root
            resp = goto_frontend_view(dummy_request)

        assert resp.location == 'http://google.com/about/'
        assert isinstance(resp, HTTPFound)

    def test_goto_frontend_not_publishable_types(self, config, root, content, filedepot):
        file_obj = self._create_file(config)
        root['file'] = file_obj
        from kotti_backend.views.goto_frontend import goto_frontend_view
        from pyramid.httpexceptions import HTTPFound
        settings = {'kotti_backend.not_publishable_types': ('File',)}
        environ = {'SCRIPT_NAME': '/cms'}
        dummy_request = self.dummy_request(root['file'], settings, environ)

        assert dummy_request.resource_url(root['file']) == 'http://example.com/cms/file/'

        import mock
        with mock.patch('kotti_backend.views.goto_frontend.get_root') as get_root:
            get_root.return_value = root
            resp = goto_frontend_view(dummy_request)

        assert resp.location == 'http://example.com/'
        assert isinstance(resp, HTTPFound)

    def test_goto_frontend_edit_links(self, root, app, config, content):
        config.include('kotti.views')
        config.include('kotti.views.edit.actions')
        config.include('kotti_backend')
        assert len([item.name for item in root['about'].type_info.edit_links
                    if hasattr(item, 'name') and item.name == 'goto_frontend']) == 1



class TestSecurity:

    def test_populate_acl(self, db_session, root):
        from kotti_backend.security import SITE_ACL
        allow_everyone_view = [item for item in SITE_ACL
                               if item[0] == 'Allow' and
                               item[1] == 'System.everyone' and
                               item[2] == ['view']]
        assert allow_everyone_view == []



import pytest
from kotti.resources import (
    File,
    Image,
    )


class TestFileImageWorkflow:

    def make_document(self, root):
        from kotti.resources import Document
        doc = root['doc'] = Document()
        from kotti import DBSession
        DBSession.flush()
        DBSession.refresh(doc)
        return doc

    @pytest.mark.parametrize("factory", [File, Image])
    def test_default_workflow_iface(self, factory, db_session, root, filedepot,
                                    config):
        from kotti.interfaces import IDefaultWorkflow
        config.include('kotti.views')  # custom predicates
        config.include('kotti_backend')
        instance = factory(data='file content',
                           name=u'content',
                           title=u'content')
        assert IDefaultWorkflow.providedBy(instance)

    def test_private_workflow(self, app, root):
        doc = self.make_document(root)
        assert doc.state == u'private'

    def test_private_workflow_name(self, app, root):
        doc = self.make_document(root)
        from kotti.workflow import get_workflow
        wf = get_workflow(doc)
        assert wf.name == u'private_workflow'



def includeme(config):  # pragma: no cover
    """ [Optional] Change the default view for the site root (@@contents).
        Useful if you want to use your kotti instance like a
        private content administration area.
    """
    config.add_route('home', '/')
    config.add_view('kotti.views.edit.actions.contents',
                    route_name=u'home',
                    permission=u'view',
                    renderer='kotti:templates/edit/contents.pt',
                    )



# -*- coding: utf-8 -*-

"""
Created on 2015-06-22
:author: Davide Moro (davide.moro@gmail.com)
"""
from zope.interface import implementer
from pyramid.settings import asbool
from pyramid.i18n import TranslationStringFactory
from kotti.interfaces import IDefaultWorkflow
from kotti.resources import (
    File,
    Image,
    Content,
    )
from kotti.util import Link


_ = TranslationStringFactory('kotti_backend')


def kotti_configure(settings):
    """ Add a line like this to you .ini file::

            kotti.configurators =
                kotti_backend.kotti_configure

        to enable the ``kotti_backend`` add-on.

    :param settings: Kotti configuration dictionary.
    :type settings: dict
    """

    settings['pyramid.includes'] += ' kotti_backend'
    settings['kotti.populators'] = 'kotti_backend.populate.populate'

    if asbool(settings.get('kotti_backend.goto_frontend')):
        frontend_link = Link(
            name='goto_frontend',
            title=_('Goto frontend'),
            target='_blank',
            )
        Content.type_info.edit_links.append(frontend_link)


def includeme(config):
    """ Don't add this to your ``pyramid_includes``, but add the
    ``kotti_configure`` above to your ``kotti.configurators`` instead.

    :param config: Pyramid configurator object.
    :type config: :class:`pyramid.config.Configurator`
    """

    # Set a default permission.
    # If you want to bypass the default permission for certain views,
    # you can decorate them with a special permission
    # (``NO_PERMISSION_REQUIRED`` from ``pyramid.security`` which indicates
    # that the view should always be executable by entirely anonymous users,
    # regardless of the default permission.
    config.set_default_permission('view')

    # Assign the default workflow for files and images
    implementer(IDefaultWorkflow)(Image)
    implementer(IDefaultWorkflow)(File)

    base_includes = ('kotti_backend.views.goto_frontend',)
    for include in base_includes:
        config.include(include)

    # translations
    config.add_translation_dirs('kotti_backend:locale')



from pytest import mark


class TestViews:

    @mark.user('admin')
    def test_root_view_admin(self, webtest):
        # admin can view
        resp = webtest.get('/')
        assert resp.status_code == 200

    def test_root_view(self, webtest):
        # view requires authentication
        resp = webtest.get('/')
        assert resp.status_code == 302

    @mark.user('admin')
    def test_root_view_admin_default(self, webtest):
        # admin can view
        resp = webtest.get('/')
        assert '/@@contents' not in resp.body
